/*
 * Copyright (C) 2000-2009 LAAS/CNRS
 *
 * Xavier BROQUERE - Mon Feb  2 2009
 *
 *
 *
 */

#include "vimanConst.h"

#ifndef _VIMANSTRUCT_H
#define _VIMANSTRUCT_H

/* max static strings length */
#ifndef VIMAN_MAX_STRLEN
#define VIMAN_MAX_STRLEN	512
#endif

#ifndef VIMAN_MAX_OBJ_NB
#define VIMAN_MAX_OBJ_NB 30
#endif

#ifndef VIMAN_MAX_MARKER_NB
#define VIMAN_MAX_MARKER_NB 15
#endif

typedef struct VimanString {
  char str[VIMAN_MAX_STRLEN];
}VimanString; 

typedef struct VimanMotionMat {
  double px, py, pz ;     /* Position */
  double qn, qi, qj, qk;  /* Orientation */
  double vx, vy, vz;      /* Linear velocity */
  double wx, wy, wz;      /* Angular velocity */
} VimanMotionMat;

typedef struct VimanCorrectionVec {
  double dx;
  double dy;
  double dz;
} VimanCorrectionVec;

typedef struct VimanPoint2D {
  double x, y;
} VimanPoint2D;

/* nx  ox  ax  pz
    ny  oy  ay  py
    nz  oz  az  pz  */
typedef struct VimanThetaMat{
  double nx, ny, nz;
  double ox, oy, oz;
  double ax, ay, az;
  double px, py, pz; /* this is the position in the frame of the base of the arm */
} VimanThetaMat;

typedef struct VimanEuler {
  double x;
  double y;
  double z;
  double yaw;
  double pitch;
  double roll;
}  VimanEuler;

typedef enum VimanBool {
  VIMAN_DISABLE,
  VIMAN_ENABLE
} VimanBool;

typedef struct VimanObject {
  unsigned long tacq_usec;	/** time of acquisition (microseconds)  of the left image */
  unsigned long tacq_sec;	/** time of acquisition (seconds) of the left image */
  VimanBool track;
  VimanBool calibrate;
  //int multitrackerId; // index in multitracker array
  int found_L; /* 1 yes 0 no*/
  int found_R; /* 1 yes 0 no*/
  char name[VIMAN_MAX_STRLEN];	/* object name */
  char file[VIMAN_MAX_STRLEN];	/* file name */
  int nbMarkers;
  int markerIDs[VIMAN_MAX_MARKER_NB];
  float patternWidths[VIMAN_MAX_MARKER_NB];
  VimanThetaMat relativePoses[VIMAN_MAX_MARKER_NB]; //position of the marker wrt the object frame
  int markerCalibrated[VIMAN_MAX_MARKER_NB]; //used for markers calibration
  double marker_visibility_L; 
  double marker_visibility_R; //visibility criterion to find the better marker to be used to locate the object
  VimanThetaMat thetaMatCam;
  VimanThetaMat thetaMatRob;
  VimanThetaMat thetaMatRob30;
  VimanMotionMat motionMat;
  // Pour la stereo
  VimanPoint2D cornersL[VIMAN_MAX_MARKER_NB][4]; //position of marker corners on left image
  VimanPoint2D cornersR[VIMAN_MAX_MARKER_NB][4]; //position of marker corners in right image
  double markerVisibilityL[VIMAN_MAX_MARKER_NB];
  double markerVisibilityR[VIMAN_MAX_MARKER_NB];
  int markerFoundL[VIMAN_MAX_MARKER_NB];
  int markerFoundR[VIMAN_MAX_MARKER_NB];
  int markerIndex_maxVisible_L;
  int markerIndex_maxVisible_R;
  double patternWidth_maxVisible_L;
  double patternWidth_maxVisible_R;
} VimanObject;

typedef struct VimanTime {
  unsigned int tacq_usec;	/** time of acquisition (microseconds)  of the left image */
  unsigned int tacq_sec;	/** time of acquisition (seconds) of the left image */
} VimanTime;
 
typedef enum VimanObjSymMotion {
  VIMAN_NONE = 0,
  VIMAN_STATIC = 1,
  VIMAN_MOVING = 2
} VimanObjSymMotion;

typedef struct VimanObjectPublic {
  unsigned long tacq_usec;	/** time of acquisition (microseconds) of the left image */
  unsigned long tacq_sec;	/** time of acquisition (seconds)  of the left image*/
  char name[VIMAN_MAX_STRLEN];	/* object name */
  int found_Left; /* 1 yes 0 no*/
  int found_Right; /* 1 yes 0 no*/
  int found_Stereo;
  VimanObjSymMotion motion; // symbolic object motion
  VimanThetaMat thetaMatRobot;
  VimanEuler eulerRobot;
  VimanThetaMat thetaMatOrigin;
  VimanEuler eulerOrigin;
  VimanThetaMat thetaMatCam;
  VimanEuler eulerOrigin30;
} VimanObjectPublic;

typedef struct VimanObjectArray {
  int nbObjects;
  VimanObject objects[VIMAN_MAX_OBJ_NB];
} VimanObjectArray;

typedef struct VimanObjectPublicArray {
  int nbObjects;
  int unused;
  VimanObjectPublic objects[VIMAN_MAX_OBJ_NB];
} VimanObjectPublicArray;

typedef enum VimanInputStream {
  VIMAN_NO_INPUT,
  VIMAN_INPUT_VIAM,
  VIMAN_INPUT_WEBCAM 
} VimanInputStream;

typedef struct VimanInitStr {
  VimanInputStream inputStream;
  VimanBool stereo;
  VimanBool color;
  char filePath[VIMAN_MAX_STRLEN]; // with the absolute path 
  char viamPosterName[100];
  char viamImageLeftName[100];
  char viamImageRightName[100];
  double stereoRatioBaseline;
  VimanBool display;
  int unused;
} VimanInitStr;



typedef struct VimanImage {
  int  nChannels;		/**< 1 or 3 channels */
  int  depth;			/**< pixel depth in bits */
  int  width;			/**< image width in pixels */
  int  height;			/**< image height in pixels */
  int  imageSize;		/**< image data size in bytes */
  unsigned char imageData[147456];	/**< pointer to aligned image data */
  int  widthStep;		/**< size of aligned image row in bytes */
} VimanImage;

#endif


